
#include <avr/interrupt.h>
#include <avr/io.h>

#include <util/delay.h>

#include <stdio.h>
#include <stdlib.h>

// Outputs
#define FLEDS PD0
#define MOTORX PD1
#define BLEDR PD4
#define BLEDG PD5
#define BLEDB PD7

// Inputs
#define BUTR PB0
#define BUTG PB1
#define BUTB PB2
#define BUTX PB3
#define BUTS PB4



#define CX 1

typedef enum {
	CLOSED,
	NEWPASS,
	OPEN
}state;

int currentRed;
int currentGreen;
int currentBlue;

int setRed;
int setGreen;
int setBlue;

state currentState;


void update_color(int r, int g, int b)
{
    OCR0A = (uint8_t)((uint32_t)(r * 7) * 0xFF / 100);
    OCR1A = (uint8_t)((uint32_t)(g * 10) * 0xFF / 100);
    OCR2A = (uint8_t)((uint32_t)(b * 10 ) * 0xFF / 100);
}


int passwordsMatch()
{
	return (currentRed == setRed) &&
			(currentGreen == setGreen) &&
			(currentBlue == setBlue );
}

void clearPassword()
{
	currentRed = 0;
	currentGreen = 0;
	currentBlue = 0;
	update_color(0,0,0);
}

void resetPassword()
{
	setRed = 0;
	setGreen = 0;
	setBlue = 0;
}

void runMotor()
{
	PORTD |=  (1 << PD1);
	PORTD |=  (2 << PD1);
	
	_delay_ms(200);
	
	PORTD &= ~(1 << PD1);
	PORTD &= ~(1 << PD1);
}

void flashLeds()
{
	PORTD &=  ~(1 << FLEDS);
		_delay_ms(300);
	PORTD ^= (1 << FLEDS);
		_delay_ms(300);
	PORTD ^= (1 << FLEDS);
		_delay_ms(300);
	PORTD ^= (1 << FLEDS);
		_delay_ms(300);
	PORTD |= (1 << FLEDS);
}


// =================================RGB
ISR(TIMER0_OVF_vect)
{
    PORTD |= (1 << PD4);
}

ISR(TIMER0_COMPB_vect)
{
    PORTD &= ~(1 << PD4);
}

ISR(TIMER1_OVF_vect)
{
    PORTD |= (1 << PD5);
}

ISR(TIMER1_COMPB_vect)
{
    PORTD &= ~(1 << PD5);
}


ISR(TIMER2_OVF_vect)
{
    PORTD |= (1 << PD7);
}

ISR(TIMER2_COMPB_vect)
{
    PORTD &= ~(1 << PD7);
}



void timer0_init()
{
    TCCR0A = (3 << WGM00) | (2 << COM0A0);
    TCCR0B = (1 << CS00);
    OCR0A = 0xC0;
}

void timer1_init()
{
    TCCR1A = (3 << WGM10) | (2 << COM1A0);
    TCCR1B = (1 << CS10);
    OCR1A = 0xC0;
}


void timer2_init()
{
    TCCR2A = (3 << WGM20) | (2 << COM2A0);
    TCCR2B = (1 << CS20);
    OCR2A = 0xC0;
}




// ==================================================================HANDLER
ISR(PCINT1_vect)
{	

	if ( currentState == NEWPASS ) {
		if((PINB & (1 << BUTR)) == 0) {
			setRed += CX;
			update_color(setRed, setGreen, setBlue);
			_delay_ms(200);

		}
		
		else if((PINB & (1 << BUTG)) == 0) {
			setGreen += CX;
			update_color(setRed, setGreen, setBlue);
			_delay_ms(200);

		}
			
		else if((PINB & (1 << BUTB)) == 0) {
			setBlue += CX;
			update_color(setRed, setGreen, setBlue);
			_delay_ms(200);
		}
			
		else if((PINB & (1 << BUTS)) == 0) {
			//runMotor(); manual close
			flashLeds();
			clearPassword();
			update_color(0, 0, 0);
			currentState = CLOSED;
		}
	}
	else if (currentState == CLOSED) {
		if((PINB & (1 << BUTR)) == 0) {
			currentRed += CX;
			update_color(currentRed, currentGreen, currentBlue);
			_delay_ms(200);
		}
		
		if((PINB & (1 << BUTG)) == 0) {
			currentGreen += CX;
			update_color(currentRed, currentGreen, currentBlue);
			_delay_ms(200);
		}
			
		if((PINB & (1 << BUTB)) == 0) {
			currentBlue += CX;
			update_color(currentRed, currentGreen, currentBlue);
			_delay_ms(200);
		}
		
		if((PINB & (1 << BUTX)) == 0) {
			clearPassword();
			update_color(currentRed, currentGreen, currentBlue);
			flashLeds();
			_delay_ms(200);
		}
			
		if ( passwordsMatch() ) {
			runMotor();
			flashLeds();
			currentState = OPEN;
		}
	}
	else if (currentState == OPEN ) {
		
		// Pressed close
		if((PINB & (1 << BUTX)) == 0) {
			runMotor();
			flashLeds();
			clearPassword();
			currentState = CLOSED;

		}
		// Pressed reset password
		else if((PINB & (1 << BUTS)) == 0) {
			PORTD &=  ~(1 << FLEDS);
			setRed = 0;
			setGreen = 0;
			setBlue = 0;
			currentState = NEWPASS;
			_delay_ms(200);
		}	
	}
}

/* initializeaza intrarile si iesirile */
void IO_init()
{
    DDRD |= (1 << PD0);
	DDRD |= (1 << PD1);	
	
	// LED-ul RGB este pus ca iesire
    DDRD |= (1 << PD4) | (1 << PD5) | (1 << PD7);
    
    // butoanele sunt puse ca intrare, cu rezistenta de pull-up
	DDRB &= ~((1 << PB0) | (1 << PB1) | (1 << PB2) | (1 << PB3) | (1 << PB4));
    PORTB |= ((1 << PB0) | (1 << PB1) | (1 << PB2) | (1 << PB3) | (1 << PB4));
	
    // intreruperile sunt activate pentru PCINT8 (PB0) si PCINT9 (PB1)
    PCICR  = (1 << PCIE1);
    PCMSK1 = (1 << PCINT8) | (1 << PCINT9) | (1 << PCINT10) | (1 << PCINT11) | (1 << PCINT12);
}



int main(void)
{
	sei();
    IO_init();
	currentState = NEWPASS;
	resetPassword();
	PORTD &=  ~(1 << FLEDS);
	PORTD &=  ~(1 << MOTORX);
	
	
	timer0_init();
	timer1_init();
	timer2_init();
	update_color(0,0,0);
	
    while(1)
    {	
        _delay_ms(1000);
    }
    
    return 0;
}
